/*************************************************************/
// Copyright (c) 2020 Oracle and/or its affiliates.
/*************************************************************/


//OCI User Configurations
variable "region" {
  type        = string
  description = "Region name where OCI resources to be created"
}

variable "tenancy_ocid" {
  type        = string
  description = "Tenancy OCID where OCI resources to be created"
}

variable "compartment_ocid" {
  type        = string
  description = "Compartment OCID where OCI resources to be created"
}

variable "user_ocid" {
  type        = string
  default     = ""
  description = "OCI User OCID"
}

variable "fingerprint" {
  type        = string
  default     = ""
  description = "OCI API key fingerprint"
}

variable "oci_api_private_key_path" {
  type        = string
  default     = ""
  description = "OCI API key file path"
}


//SBC General Configurations

variable "prefix" {
  type        = string
  description = "Added as prefix to display name of each OCI resource being created, make sure to be unique for each OCI RM stack"
}

variable "bucket_name" {
  type        = string
  description = "OCI object storage bucket to store SBC's metadata and userdata"
}

/****************************************
Allowed values are
       Session_Border_Controller
       Enterprise_Session_Border_Controller
       Peering_Session_Border_Controller
       Subscriber_Aware_Load Balancer
       Session_Router_Session_Stateful
       Session_Router_Transaction_Stateful
****************************************/
variable "product_type" {
  type        = string
  description = "SD product type, for allowed values see README"
}

variable "high_availability" {
  type        = bool
  default     = true
  description = "Enable or Disable Oracle Session Delivery High Availability, true: HA-pair and false: Standalone"
}

variable "enable_rest_interface" {
  type        = bool
  default     = true
  description = "Enable or Disable REST interface, use self-signed certificate to enable REST Interface"
}

/******************************************************************
# Enable or Disable provisioning of a minimal base configuration for the VM.
# true: A base configuration will be applied to the VM which is
#       suitable for minimal Standalone or HA-pair functionality.
#       Note that a full service configuration must be provisioned
#       separately after the VM boots.
# false: No configuration will be applied to the VM. The VM will
#        boot without any configuration elements provisioned.
******************************************************************/
variable "apply_base_config" {
  type        = bool
  default     = true
  description = "Enable or Disable provisioning of a minimal base configuration for the VM"
}

variable "licenses" {
  type        = string
  default     = ""
  description = "Comma seperated list of license keys, for example \"ABCDEFGH4321, abcdefgh1234\""
}

variable "user_pass" {
  type        = string
  default     = ""
  description = "Password (SHA1 encoded) for the 'user' account"
}

variable "admin_pass" {
  type        = string
  default     = ""
  description = "Password (SHA1 encoded) for the 'admin' account"
}


//Instance Configurations
variable "sbc_image_ocid" {
  type        = string
  default     = ""
  description = "OCID of the SBC image"
}
/***************************************************************
A shape is a template that determines the number of CPUs, amount of memory, and other resources allocated to a newly created instance.
Currently SBC supports
        - VM.Standard2.4
        - VM.Standard2.8
        - VM.Standard2.16
        - VM.Standard2.24
***************************************************************/
variable "instance_shape" {
  type        = string
  description = "A shape is a template that determines the number of CPUs, amount of memory, and other resources allocated to a newly created instance"
}

/***************************************************************************
# If SSH public key provided by user,
# SBC stack also creates a instance serial console connnection
***************************************************************************/
variable "ssh_public_key" {
  type        = string
  default     = ""
  description = "SSH public key contents, User can login to SBC without password prompt"
}

variable "boot_volume_size_in_gbs" {
  type        = number
  default     = 50
  description = "Boot volume size in GB, min = 50 and max 16384"
}

variable "availability_domain" {
  type        = string
  description = "The Availability Domain name in which SD VM(s) should be placed"
}

//In case of HA pair, two FDs are automatically selected
variable "fault_domain" {
  type        = string
  default     = ""
  description = "The Fault Domain name in which SD VM should be placed, ignored for HA deployments"
}


//vNIC Configurations

//wancom0
variable "wancom0_subnet_id" {
  type        = string
  description = "The OCID of the subnet wancom0 with which the instance primary VNIC is created"
}

variable "wancom0_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom0 interface (standalone or primary node)"
}

variable "wancom0_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom0 interface (secondary node)"
}

//wancom1
variable "wancom1_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet wancom1 with which the instance secondary VNIC is created"
}

variable "wancom1_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom1 interface (standalone or primary node)"
}

variable "wancom1_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom1 interface (secondary node)"
}

//wancom2
variable "wancom2_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet wancom2 with which the instance secondary VNIC is created"
}

variable "wancom2_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom2 interface (standalone or primary node)"
}

variable "wancom2_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for Wancom2 interface (secondary node)"
}

//s0p0
variable "s0p0_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet s0p0 with which the instance secondary VNIC is created"
}

variable "s0p0_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s0p0 interface (standalone or primary node)"
}

variable "s0p0_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s0p0 interface (secondary node)"
}

variable "s0p0_virtual_ips" {
  type        = string
  default     = ""
  description = "List of static virtual IPs for media interface s0p0, ex: \"10.3.0.10, 10.3.0.11\""
}

variable "s0p0_num_vips" {
  type        = number
  default     = 1
  description = "Number of virtual IPs to be created, ignored if static virtual IPs are provided"
}

variable "s0p0_assign_public_ip" {
  type        = bool
  default     = false
  description = "Assign a reserved public IP to the s0p0 interface"
}


//s1p0
variable "s1p0_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet s1p0 with which the instance secondary VNIC is created"
}

variable "s1p0_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s1p0 interface (standalone or primary node)"
}

variable "s1p0_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s1p0 interface (secondary node)"
}

variable "s1p0_virtual_ips" {
  type        = string
  default     = ""
  description = "List of static virtual IPs for media interface s1p0, ex: \"10.4.0.10, 10.4.0.11\""
}

variable "s1p0_num_vips" {
  type        = number
  default     = 1
  description = "Number of virtual IPs to be created, ignored if static virtual IPs are provided"
}

variable "s1p0_assign_public_ip" {
  type        = bool
  default     = false
  description = "Assign a reserved public IP to the s1p0 interface"
}


//s0p1
//The OCID of the subnet s0p1 with which the instance secondary VNIC is created
variable "s0p1_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet wancom2 with which the instance secondary VNIC is created"
}

//Static IP for s0p1 interface (standalone or primary node)
variable "s0p1_primary_node_ip" {
  type        = string
  default     = ""
  description = "Network Time Protocol server 2 to use for time synchronization"
}

//Static IP for s0p1 interface (secondary node)
variable "s0p1_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Network Time Protocol server 2 to use for time synchronization"
}

variable "s0p1_virtual_ips" {
  type        = string
  default     = ""
  description = "List of static virtual IPs for media interface s0p1, ex: \"10.5.0.10, 10.5.0.11\""
}

variable "s0p1_num_vips" {
  type        = number
  default     = 1
  description = "Number of virtual IPs to be created, ignored if static virtual IPs are provided"
}

variable "s0p1_assign_public_ip" {
  type        = bool
  default     = false
  description = "Assign a reserved public IP to the s0p1 interface"
}


//s1p1
variable "s1p1_subnet_id" {
  type        = string
  default     = ""
  description = "The OCID of the subnet s1p1 with which the instance secondary VNIC is created"
}

variable "s1p1_primary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s1p1 interface (standalone or primary node)"
}

variable "s1p1_secondary_node_ip" {
  type        = string
  default     = ""
  description = "Static IP for s1p1 interface (secondary node)"
}

variable "s1p1_virtual_ips" {
  type        = string
  default     = ""
  description = "List of static virtual IPs for media interface s1p1, ex: \"10.6.0.10, 10.6.0.11\""
}

variable "s1p1_num_vips" {
  type        = number
  default     = 1
  description = "Number of virtual IPs to be created, ignored if static virtual IPs are provided"
}

variable "s1p1_assign_public_ip" {
  type        = bool
  default     = false
  description = "Assign a reserved public IP to the s1p1 interface"
}


//CPU core Configurations
variable "forwarding_cores" {
  type        = number
  default     = 1
  description = "The number of CPU cores dedicated for forwarding frames"
}

variable "dos_cores" {
  type        = number
  default     = 0
  description = "The number of CPU cores dedicated for Denial-of-service protection"
}

variable "transcoding_cores" {
  type        = number
  default     = 0
  description = "The number of CPU cores dedicated for transcoding media"
}

variable "use_sibling_cores" {
  type        = bool
  default     = false
  description = "Specify if sibling core for datapath is to be used, if SMT topology is aware to vSBC"
}


//SNMP Configurations
variable "snmp_community_name" {
  type        = string
  default     = ""
  description = "Name of the SNMPv2 community to use for SNMP management"
}

variable "snmp_ip_address" {
  type        = string
  default     = ""
  description = "An IP address to add to the SNMPv2 community for SNMP management"
}


//NTP Configurations
variable "ntp_server1" {
  type        = string
  default     = ""
  description = "Network Time Protocol server 1 to use for time synchronization"
}
variable "ntp_server2" {
  type        = string
  default     = ""
  description = "Network Time Protocol server 2 to use for time synchronization"
}


//Disk Partitions
variable "create_disk_partitions" {
  type        = bool
  default     = true
  description = "Create default disk partitions /app and /sys with 20% and 80% ratio of memory"
}

//Validate input parameters
locals {
  //regex pattern for single ipv4 address
  regex_ipv4 = "^$|^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])$"

  //regex pattern for comma seperated multiple ipv4 address
  regex_ipv4_multi = "^$|^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])(?:,\\s*(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9]))*$"

  validate = var.high_availability == false ? (var.wancom0_subnet_id == "" ? -1 : 1) : (var.wancom1_subnet_id == "" && var.wancom2_subnet_id == "") ? -1 : 1


  val_s0p0_primary_node_ip = regex(local.regex_ipv4, var.s0p0_primary_node_ip)
  val_s1p0_primary_node_ip = regex(local.regex_ipv4, var.s1p0_primary_node_ip)
  val_s0p1_primary_node_ip = regex(local.regex_ipv4, var.s0p1_primary_node_ip)
  val_s1p1_primary_node_ip = regex(local.regex_ipv4, var.s1p1_primary_node_ip)

  val_wancom0_primary_node_ip = regex(local.regex_ipv4, var.wancom0_primary_node_ip)
  val_wancom1_primary_node_ip = regex(local.regex_ipv4, var.wancom1_primary_node_ip)
  val_wancom2_primary_node_ip = regex(local.regex_ipv4, var.wancom2_primary_node_ip)


  val_s0p0_secondary_node_ip = regex(local.regex_ipv4, var.s0p0_secondary_node_ip)
  val_s1p0_secondary_node_ip = regex(local.regex_ipv4, var.s1p0_secondary_node_ip)
  val_s0p1_secondary_node_ip = regex(local.regex_ipv4, var.s0p1_secondary_node_ip)
  val_s1p1_secondary_node_ip = regex(local.regex_ipv4, var.s1p1_secondary_node_ip)

  val_wancom0_secondary_node_ip = regex(local.regex_ipv4, var.wancom0_secondary_node_ip)
  val_wancom1_secondary_node_ip = regex(local.regex_ipv4, var.wancom1_secondary_node_ip)
  val_wancom2_secondary_node_ip = regex(local.regex_ipv4, var.wancom2_secondary_node_ip)

  val_snmp_ip_address = regex(local.regex_ipv4, var.snmp_ip_address)
  val_ntp_server1     = regex(local.regex_ipv4, var.ntp_server1)
  val_ntp_server2     = regex(local.regex_ipv4, var.ntp_server2)


  val_s0p0_virtual_ips = regex(local.regex_ipv4_multi, var.s0p0_virtual_ips)
  val_s1p0_virtual_ips = regex(local.regex_ipv4_multi, var.s1p0_virtual_ips)
  val_s0p1_virtual_ips = regex(local.regex_ipv4_multi, var.s0p1_virtual_ips)
  val_s1p1_virtual_ips = regex(local.regex_ipv4_multi, var.s1p1_virtual_ips)

}
